using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Domain;
using CommonEntity;

namespace Service
{
    public interface ISysHotelManager : IGenericManager<SysHotel>
	{
        IList<SysHotel> LoadAllByPage(out long total, int page, int rows, string order, string sort, string query, int? groupId);
        IList<SysHotel> LoadFCS_PushData();

        SysHotel GetByCode(string code);

        SysHotel GetByCode(string code, DateTime date);

        SysHotel GetByAssociatedAccount(string associatedAccount);

        SysHotel GetByDomainUrl(string domainUrl);
        SysHotel GetByETV_HotelId(string ETVHotelID);

        void UpdateDayTime(SysHotel entity);


        List<CommonEntity.KongQi> GetNeedData();

        List<SomeDeviceExistsData> IsExistsSomeDeviceRoomModal(int id, string roomnum, string start_address, string end_address);

        List<SysHotel> get_fcs_hotel();
	}
}
